{
Client Access Objects 1.5

Catalog demo using TcaoDatabaseCatalog

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

Make sure you change the systemName of the TcaoDatabaseConnection component to
your system name.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit CatalogMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, ComCtrls,
  caoDatabaseCatalog, caoDatabaseDataFormat, caoDatabaseConnection,
  caoErrorMessage, caoCommonUtils,

  // Import type library "Client Access Object Library" before compiling.

  Cwbautsv_TLB;

type
  TMainForm = class(TForm)
    StatusLine: TStatusBar;
    TreeView1: TTreeView;
    caoErrorMessage1: TcaoErrorMessage;
    caoDatabaseDataFormat1: TcaoDatabaseDataFormat;
    caoDatabaseCatalog1: TcaoDatabaseCatalog;
    caoDatabaseConnection1: TcaoDatabaseConnection;
    procedure LoadCollection;
    procedure GetLibraries(Lib: string);
    procedure GetFiles(Lib,FileName: string);
    procedure GetFields(Lib,FileName: string);
    procedure LibToTree;
    procedure FileToTree(RootNode: TTreeNode);
    procedure FieldToTree(RootNode: TTreeNode);
    procedure FormCreate(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TreeView1Expanded(Sender: TObject; Node: TTreeNode);
  private
    FieldInfoColl: TCollection;
    FieldDataColl: TCollection;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

{
Load collection with last request.
}

procedure TMainForm.LoadCollection;
var
  DataLen: integer;
  DataBuf: OleVariant;
  IndicatorCount: integer;
  IndicatorBuf: OleVariant;
  I: integer;
  S: string;
begin
  with caoDatabaseCatalog1 do
  begin
    DataLen := resultDataLength;
    if DataLen > 0 then
    begin
      GetFieldInfo(caoDatabaseCatalog1,FieldInfoColl);
      DataBuf := VarArrayCreate([0,DataLen-1],varByte);
      IndicatorCount := resultIndicatorCount;
      IndicatorBuf := VarArrayCreate([0,IndicatorCount-1],varSmallint);
      GetResultData(DataBuf,DataLen,IndicatorBuf,IndicatorCount);
      S := VarByteToStr(DataBuf);
      I := 1;
      FieldDataColl.Clear;
      while I < DataLen do
        I := GetFieldData(S,I,FieldInfoColl,FieldDataColl)
    end
  end
end;

{
Get libraries.
}

procedure TMainForm.GetLibraries(Lib: string);
begin
  with caoDatabaseCatalog1 do
  begin
    RetrieveLibraryInfo(Lib,cwbdbGetLibraryName or
    cwbdbGetLibraryDesc or cwbdbGetLibraryOwner);
    LoadCollection
  end
end;

{
Get files.
}

procedure TMainForm.GetFiles(Lib,FileName: string);
begin
  with caoDatabaseCatalog1 do
  begin
    RetrieveFileInfo(Lib,FileName,cwbdbGetFileName);
    LoadCollection
  end
end;

{
Get fields.
}

procedure TMainForm.GetFields(Lib,FileName: string);
begin
  with caoDatabaseCatalog1 do
  begin
    RetrieveFieldInfo(Lib,FileName,'*',cwbdbGetFldName);
    LoadCollection
  end
end;

{
List libraries.
}

procedure TMainForm.LibToTree;
var
  I: Integer;
  S: string;
  RecCnt: Integer;
  RootNode: TTreeNode;
begin
  with TreeView1.Items do
  begin
    Clear; // remove any existing nodes
    RootNode := nil;
    RecCnt := FieldDataColl.Count div 3;
    for I := 0 to RecCnt-1 do
    begin
      RootNode := Add(RootNode,
      Trim(T400FieldData(FieldDataColl.Items[I*3]).Data));
      S := Trim(T400FieldData(FieldDataColl.Items[I*3+1]).Data);
      AddChild(RootNode,S);
      S := Trim(T400FieldData(FieldDataColl.Items[I*3+2]).Data);
      if S = '' then
        S := 'No description';
      AddChild(RootNode,S)
    end
  end
end;

{
List libraries.
}

procedure TMainForm.FileToTree(RootNode: TTreeNode);
var
  I: Integer;
  S: string;
  ChildNode: TTreeNode;
begin
  with TreeView1.Items do
  begin
    for I := 0 to FieldDataColl.Count-1 do
    begin
      S := Trim(T400FieldData(FieldDataColl.Items[I]).Data);
      ChildNode := AddChild(RootNode,S);
      AddChild(ChildNode,'Fields');
    end
  end
end;

{
List fields.
}

procedure TMainForm.FieldToTree(RootNode: TTreeNode);
var
  I: Integer;
  S: string;
  RecCnt: Integer;
begin
  with TreeView1.Items do
  begin
    RecCnt := FieldDataColl.Count;
    for I := 0 to RecCnt-1 do
    begin
      S := Trim(T400FieldData(FieldDataColl.Items[I]).Data);
      AddChild(RootNode,S);
    end
  end
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Application.OnHint := ShowHint;
end;

procedure TMainForm.ShowHint(Sender: TObject);
begin
  StatusLine.SimpleText := Application.Hint;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  // Initilize collections
  FieldInfoColl := TCollection.Create(T400FieldInfo);
  FieldDataColl := TCollection.Create(T400FieldData);
  // Connect to database and get all libraries
  caoErrorMessage1.Setup;
  caoDatabaseConnection1.Active := True;
  caoDatabaseCatalog1.Setup;
  GetLibraries('*ALL');
  LibToTree
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Free collections and disconnect database
  FieldInfoColl.Free;
  FieldDataColl.Free;
  caoDatabaseConnection1.Active := False
end;

{
Handle dynamically generated nodes.
}

procedure TMainForm.TreeView1Expanded(Sender: TObject; Node: TTreeNode);
var
  ChildNode: TTreeNode;
begin
  with TreeView1.Items do
  begin
    // Add files
    if (Node.Count < 3) and (Node.Parent = nil) then
    begin
      ChildNode := AddChild(Node,'Files');
      GetFiles(Node.Text,'*ALL');
      FileToTree(ChildNode)
    end
    else
      // Add fields
      if (Node.Item[0].Text = 'Fields') and (Node.Item[0].Count = 0) then
      begin
        GetFields(TTreeNode(Node.Parent).Parent.Text,Node.Text);
        FieldToTree(Node.Item[0])
      end
  end
end;

end.
