{
Client Access Objects 1.5

caoCommand component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbCommand automation object is used to run a command on the AS/400 system.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoCommand;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage, caoRemoteCallConversation,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoCommand exception object

  EcaoCommand = class(Exception);

  // TcaoCommand component

  TcaoCommand = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Integer;
    FcommandString: String;
    FcwbCommand: cwbCommand;
    FcaoErrorMessage: TcaoErrorMessage;
    FcaoRemoteCallConversation: TcaoRemoteCallConversation;
    FOnRunCmd: TNotifyEvent;
  protected
    { Protected declarations }
    function ResultCodeText: String; virtual;
    function GetErrMsg: String; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure RunCmd; virtual;
  published
    { Published declarations }
    property ReturnCode: Integer read FReturnCode default cwbOK;
    property commandString: String read FcommandString write FcommandString;
    property system: TcaoRemoteCallConversation read FcaoRemoteCallConversation write FcaoRemoteCallConversation;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property OleObject: cwbCommand read FcwbCommand;
    property OnRunCmd: TNotifyEvent read FOnRunCmd write FOnRunCmd;
  end;

implementation

{
Create OLE object.
}

constructor TcaoCommand.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbCommand := CocwbCommand.Create
end;

{
Return formatted result code text.
}

function TcaoCommand.ResultCodeText: String;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoCommand.GetErrMsg: String;
var
  S: String;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
Handle caoErrorMessage or caoRemoteCallConversation component being removed.
}

procedure TcaoCommand.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FcaoErrorMessage then
      FcaoErrorMessage := nil
    else
      if AComponent = FcaoRemoteCallConversation then
        FcaoRemoteCallConversation := nil
  end
end;

{
This method issues an AS/400 command on the specified system.

Parameters:

system - Object

A cwbRemoteCallConversation object that identifies the AS/400 system on which
the command is to be run.  Before cwbrcRunCmd is called, the
cwbrcStartConversation method must be called on the system object.

commandString - String

Pointer to a String that contains the command to be issued on the AS/400
system.  This is an ASCIIZ String.

errorMessage - Object

A cwbErrorMessage  object which is used to hold any error messages.  If the
command fails, the properties and methods of the errorMessage object can be
used to determine what went wrong.  Before cwbrcRunCmd is called, the
cwbsvSetup method must be called on the errorMessage object.
}

procedure TcaoCommand.RunCmd;
begin
  if (FcaoRemoteCallConversation <> nil) and
  (FcaoErrorMessage  <> nil) then
    FReturnCode := FcwbCommand.cwbrcRunCmd(FcaoRemoteCallConversation.OleObject,
    FcommandString,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoCommand.Create(GetErrMsg);
  if Assigned(OnRunCmd) then
    OnRunCmd(Self)
end;

end.
