{
Client Access Objects 1.5

Common functions shared among components and demos.

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoCommonUtils;

interface

uses
  Classes, ComCtrls, SysUtils, caoDatabaseCatalog,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  T400FieldInfo = class(TCollectionItem)
  public
    ColName: String;
    ColType: SmallInt;
    ColLength: Integer;
    ColPerc: SmallInt;
    ColScale: SmallInt;
    ColCCSID: SmallInt;
    ColOffset: Integer;
  end;

  T400FieldData = class(TCollectionItem)
  public
    Data: Variant;
  end;

function StrToVarByte(S: string): OleVariant;
function VarByteToStr(V: OleVariant): string;
function GetFieldInfo(DbCat: TcaoDatabaseCatalog; FldColl: TCollection): Integer;
function GetFieldData(const S: string; StartPos: integer; FldInfColl,
FldDataColl: TCollection): integer;

implementation

{
Convert string to byte array (safe array).
}

function StrToVarByte(S: string): OleVariant;
var
  I: integer;
  Len: integer;
  V: OleVariant;
begin
  Len := Length(S);
  V := VarArrayCreate([0,Len-1],varByte);
  for I := 1 to Len do
    V[I-1] := byte(S[I]);
  Result := V
end;

{
Convert byte array (safe array) to string.
}

function VarByteToStr(V: OleVariant): string;
var
  I: integer;
  Len: integer;
  S: string;
begin
  Len := VarArrayHighBound(V,1)+1;
  SetLength(S,Len);
  for I := 1 to Len do
    S[I] := char(byte(V[I-1]));
  Result := S
end;

{
Get AS/400 specific field definitions and return record length.
}

function GetFieldInfo(DbCat: TcaoDatabaseCatalog; FldColl: TCollection): Integer;
var
  I: Integer;
  Cols: Integer;
  ColOfs: Integer;
  ColType: SmallInt;
  ColName: WideString;
  ColLength: Integer;
  ColPerc: SmallInt;
  ColScale: SmallInt;
  ColCCSID: SmallInt;
  FieldDef400: T400FieldInfo;
begin
  ColOfs := 0;
  with DbCat do
  begin
    GetResultDataFormat;
    with databaseDataFormat do
    begin
      FldColl.Clear;
      Cols := columnCount;
      for I := 1 to Cols do
      begin
        FieldDef400 := T400FieldInfo(FldColl.Add);
        GetColumnName(I,ColName);
        FieldDef400.ColName := ColName;
        GetColumnType(I,ColType);
        FieldDef400.ColType := ColType;
        GetColumnLength(I,ColLength);
        FieldDef400.ColLength := ColLength;
        GetColumnPrecision(I,ColPerc);
        FieldDef400.ColPerc := ColPerc;
        GetColumnScale(I,ColScale);
        FieldDef400.ColScale := ColScale;
        GetColumnCCSID(I,ColCCSID);
        FieldDef400.ColCCSID := ColCCSID;
        FieldDef400.ColOffset := ColOfs;
        ColOfs := ColOfs+ColLength
      end
    end
  end;
  Result := ColOfs
end;

{
Get field data.
}

function GetFieldData(const S: string; StartPos: integer; FldInfColl,
FldDataColl: TCollection): integer;
var
  I: integer;
  Cols: integer;
  FieldInfo: T400FieldInfo;
  FieldData: T400FieldData;
  LongVar: integer;
  ShortVar: SmallInt;
  SingleVar: Single;
  DoubleVar: Double;
  PackedStr: string;
  P: integer;
  Pos:Integer;
  EndPos: integer;
begin
  Result := -1;
  Pos := StartPos;
  Cols := FldInfColl.Count;
  for I := 0 to Cols-1 do
  begin
    FieldInfo := T400FieldInfo(FldInfColl.Items[I]);
    with FieldInfo do
    begin
      FieldData := T400FieldData(FldDataColl.Add);
      case ColType of
        cwbdbPCNoConversion:
          FieldData.Data := Copy(S,Pos,ColLength);
        cwbdbPCString:
          FieldData.Data := Copy(S,Pos,ColLength);
        cwbdbPCLong:
          begin
            Move(S[Pos],LongVar,SizeOf(LongVar));
            FieldData.Data := LongVar
          end;
        cwbdbPCShort:
          begin
            Move(S[Pos],ShortVar,SizeOf(ShortVar));
            FieldData.Data := ShortVar
          end;
        cwbdbPCFloat:
          begin
            Move(S[Pos],SingleVar,SizeOf(SingleVar));
            FieldData.Data := SingleVar
          end;
        cwbdbPCDouble:
          begin
            Move(S[Pos],DoubleVar,SizeOf(DoubleVar));
            FieldData.Data := DoubleVar
          end;
        cwbdbPCPacked:
          begin
            PackedStr := '';
            EndPos := Pos+ColLength-1;
            for P := Pos to EndPos do
            begin
              // Convert left nibble (4 bits)
              PackedStr := PackedStr+char((byte(S[P]) and $f0) shr 4+48);
              // Convert right nibble if not last byte
              if P <> EndPos then
                PackedStr := PackedStr+char((byte(S[P]) and $0f)+48)
            end;
            // Add decimal place holder if needed
            if ColScale > 0 then
              SYSTEM.Insert('.',PackedStr,Length(PackedStr)-ColScale+1);
            // Add sign if needed
            if (byte(S[EndPos]) and $f0) = $d0 then
              PackedStr := '-'+PackedStr;
            FieldData.Data := PackedStr
          end;
        cwbdbPCZoned:
          begin
            PackedStr := Copy(S,Pos,ColLength);
            // Add sign if needed
            if (byte(PackedStr[ColLength]) and $40) = $40 then
            begin
              byte(PackedStr[ColLength]) :=
              byte(PackedStr[ColLength]) and $3F;
              PackedStr := '-'+PackedStr
            end;
            // Add decimal place holder if needed
            if ColScale > 0 then
              SYSTEM.Insert('.',PackedStr,Length(PackedStr)-ColScale+1);
            FieldData.Data := PackedStr
          end;
        cwbdbPCInvalidType:
          FieldData.Data := 'Invalid Type';
        cwbdbPCVarstring:
          begin
            Move(S[Pos],ShortVar,SizeOf(ShortVar));
            FieldData.Data := Copy(S,Pos+2,ShortVar)
          end;
        cwbdbPCGraphic:
          FieldData.Data := Copy(S,Pos,ColLength);
        cwbdbPCVargraphic:
          begin
            Move(S[Pos],ShortVar,SizeOf(ShortVar));
            FieldData.Data := Copy(S,Pos+2,ShortVar)
          end
      end;
      Pos := Pos+ColLength
    end
  end;
  Result := Pos
end;

end.
