{
Client Access Objects 1.5

caoDatabaseParmMarkerFormat component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbDatabaseParmMarkerFormat automation object holds the description of the
SQL parameter marker data.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDatabaseParmMarkerFormat;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDatabaseParmMarkerFormat exception object

  EcaoDatabaseParmMarkerFormat = class(Exception);

  TcaoPmSystemLocation = (
    caodbSystem,
    caodbLocal);

  TcaoDatabaseParmMarkerFormat = class(TComponent)
  private
    { Private declarations }
    FReturnCode: integer;
    FcwbDatabaseParmMarkerFormat: cwbDatabaseParmMarkerFormat;
    FcaoErrorMessage: TcaoErrorMessage;
    function GetParameterCount: integer;
    function GetLocation: TcaoPmSystemLocation;
    procedure SetLocation(Loc: TcaoPmSystemLocation);
    function GetSizeOfInputParameters: integer;
    function GetSizeOfOutputParameters: integer;
  protected
    { Protected declarations }
    function ResultCodeText: string; virtual;
    function GetErrMsg: string; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure GetParameterCCSID(parameterPosition: Integer; var dataCCSID: Smallint); virtual;
    procedure GetParameterLength(parameterPosition: Integer; var dataLength: Integer); virtual;
    procedure GetParameterPrecision(parameterPosition: Integer; var dataPrecision: Smallint); virtual;
    procedure GetParameterScale(parameterPosition: Integer; var dataScale: Smallint); virtual;
    procedure GetParameterType(parameterPosition: Integer; var dataType: Smallint); virtual;
    procedure IsParameterInput(parameterPosition: Integer; var parameterIsInput: WordBool); virtual;
    procedure IsParameterInputOutput(parameterPosition: Integer; var parmIsInputOutput: WordBool); virtual;
    procedure SetClientParmToNumeric(parameterPosition: Integer; parameterType: Smallint; parameterLength: Integer; parameterPrecision, parameterScale: Smallint); virtual;
    procedure SetClientParmToString(parameterPosition: Integer; parameterType: Smallint; parameterLength: Integer; parameterCCSID: Smallint); virtual;
  published
    { Published declarations }
    property ReturnCode: integer read FReturnCode default cwbOK;
    property parameterCount: integer read GetParameterCount default 0;
    property location: TcaoPmSystemLocation read GetLocation write SetLocation default caodbLocal;
    property sizeOfInputParameters: integer read GetSizeOfInputParameters default 0;
    property sizeOfOutputParameters: integer read GetSizeOfOutputParameters default 0;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property OleObject: cwbDatabaseParmMarkerFormat read FcwbDatabaseParmMarkerFormat;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDatabaseParmMarkerFormat.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDatabaseParmMarkerFormat := CocwbDatabaseParmMarkerFormat.Create
end;

{
Return formatted result code text.
}

function TcaoDatabaseParmMarkerFormat.ResultCodeText: string;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoDatabaseParmMarkerFormat.GetErrMsg: string;
var
  S: string;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
This property contains the number of parameters that are described by the
parameter marker format.  This is a read-only property.
}

function TcaoDatabaseParmMarkerFormat.GetParameterCount: integer;
begin
  Result := FcwbDatabaseParmMarkerFormat.parameterCount
end;

{
This property indicates whether client or host information is to be retrieved.
Possible values are:  cwbdbSystem and cwbdbLocal.  If this property is not
set, cwbdbLocal is used.  This is a read/write property.
}

function TcaoDatabaseParmMarkerFormat.GetLocation: TcaoPmSystemLocation;
begin
  case FcwbDatabaseParmMarkerFormat.location of
    cwbdbSystem:
      Result := caodbSystem;
    cwbdbLocal:
      Result := caodbLocal
    else
      Result := caodbLocal
  end
end;

{
Set OLE property location.
}

procedure TcaoDatabaseParmMarkerFormat.SetLocation(Loc: TcaoPmSystemLocation);
begin
  case Loc of
    caodbSystem:
      FcwbDatabaseParmMarkerFormat.location := cwbdbSystem;
    caodbLocal:
      FcwbDatabaseParmMarkerFormat.location := cwbdbLocal
  end
end;

{
This property contains the size (in bytes) of the input data described by the
parameter marker format.  This is a read-only property.
}

function TcaoDatabaseParmMarkerFormat.GetSizeOfInputParameters: integer;
begin
  Result := FcwbDatabaseParmMarkerFormat.sizeOfInputParameters
end;

{
This property contains the size (in bytes) of the output data described by the
parameter marker format.  This is a read-only property.
}

function TcaoDatabaseParmMarkerFormat.GetSizeOfOutputParameters: integer;
begin
  Result := FcwbDatabaseParmMarkerFormat.sizeOfOutputParameters
end;

{
Handle caoErrorMessage component being removed.
}

procedure TcaoDatabaseParmMarkerFormat.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and
  (AComponent = FcaoErrorMessage) then
      FcaoErrorMessage := nil
end;

{
This method returns the Coded Character Set Identifier (CCSID) for a specified
parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
dataCCSID - Pointer to Short Integer
Points to a buffer to store the CCSID for the specified parameter.
}

procedure TcaoDatabaseParmMarkerFormat.GetParameterCCSID(parameterPosition: Integer; var dataCCSID: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbGetParameterCCSID(
  parameterPosition,dataCCSID,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns the length (in bytes) of the data for a specified parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
datalength - Pointer to Short Integer
Points to a buffer to store the data length.
}

procedure TcaoDatabaseParmMarkerFormat.GetParameterLength(parameterPosition: Integer; var dataLength: Integer);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbGetParameterLength(
  parameterPosition,dataLength,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns the precision for a specified parameter.  (The total number
of binary or decimal digits of a number, excluding the sign.)

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
dataPrecision - Pointer to Short Integer
Pointer to a buffer to store the data precison.
}

procedure TcaoDatabaseParmMarkerFormat.GetParameterPrecision(parameterPosition: Integer; var dataPrecision: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbGetParameterPrecision(
  parameterPosition,dataPrecision,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns the scale (the number of digits in the fractional part of a
packed or zoned decimal number) for a specified parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
dataScale - Pointer to Short Integer
Points to a buffer to store the data scale.
}

procedure TcaoDatabaseParmMarkerFormat.GetParameterScale(parameterPosition: Integer; var dataScale: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbGetParameterScale(
  parameterPosition,dataScale,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns the data type for a specified parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This is a one-based position.
dataType - Pointer to Short Integer
Pointer to a buffer to store the data type.  Possible values are:

	cwbdbPCNoConverstion
	cwbdbPCString
	cwbdbPCShort
	cwbdbPCFloat
	cwbdbPCDouble
	cwbdbPCPacked
	cwbdbPCZoned
	cwbdbPCInvalidType
	cwbdbPCVarstring
	cwbdbPCGraphic
	cwbdbPCVargraphic
}

procedure TcaoDatabaseParmMarkerFormat.GetParameterType(parameterPosition: Integer; var dataType: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbGetParameterType(
  parameterPosition,dataType,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns a Boolean value indicating if the parameter is input only.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
parmIsInput - Pointer to Boolean
Points to a buffer to store the input-only flag.
}

procedure TcaoDatabaseParmMarkerFormat.IsParameterInput(parameterPosition: Integer; var parameterIsInput: WordBool);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbIsParameterInput(
  parameterPosition,parameterIsInput,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method returns a Boolean value indicating if the parameter is both input
and output.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
parmIsInputOutput - Pointer to Boolean
Points to a buffer to store the input and output flag.
}

procedure TcaoDatabaseParmMarkerFormat.IsParameterInputOutput(parameterPosition: Integer; var parmIsInputOutput: WordBool);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbIsParameterInputOutput(
  parameterPosition,parmIsInputOutput,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method provides a way to treat a string parameter as if it were a numeric
parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
parameterType - Short Integer
Specifies the numeric type to be used.  Possible values are:

	cwbdbLong
	cwbdbShort
	cwbdbFloat
	cwbdbDouble
	cwbdbPacked
	cwbdbZoned

parameterLength - Long
If the columType is cwbdbPacked or cwbdbZoned, it specifies the number of bytes
to be used for the parameter length.  Otherwise, this parameter is ignored.
parameterPrecision - Short Integer
If the parameterType is cwbdbPacked or cwbdbZoned, it specifies the total
number digits (excluding the sign), to be used for the parameter.  Otherwise,
this parameter is ignored.
parameterScale - Short Integer
If the parameterType is cwbdbPacked or cwbdbZoned, it specifes the number of
digits in the fractional part of the number, to be used for the parameter.
Otherwise, this parameter is ignored.
}

procedure TcaoDatabaseParmMarkerFormat.SetClientParmToNumeric(parameterPosition: Integer; parameterType: Smallint; parameterLength: Integer; parameterPrecision, parameterScale: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbSetClientParmToNumeric(
  parameterPosition,parameterType,parameterLength,parameterPrecision,
  parameterScale,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

{
This method provides a way to treat a numeric parameter as if it were a string
parameter.

Parameters:
parameterPosition - Long
Specifies the relative position of the parameter.  This position is one-based.
parameterType - Short Integer
Specifies the string type to be used.  Possible values are:

	cwbdbString
	cwbdbVarstring
	cwbdbGraphic
	cwbdbVargraphic

parameterLength - Long
Specifies the number of bytes to be used for the parameter length.
parameterCCSID - Short Integer
Specifies the parameter CCSID (coded Character Set Identifier) to be used.
}

procedure TcaoDatabaseParmMarkerFormat.SetClientParmToString(parameterPosition: Integer; parameterType: Smallint; parameterLength: Integer; parameterCCSID: Smallint);
begin
  FReturnCode := FcwbDatabaseParmMarkerFormat.cwbdbSetClientParmToString(
  parameterPosition,parameterType,parameterLength,parameterCCSID,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseParmMarkerFormat.Create(ResultCodeText)
end;

end.
