{
Client Access Objects 1.5

caoDatabaseSQL component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbDatabaseSQL automation object is used to request SQL services from an
AS/400 system.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoDatabaseSQL;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage, caoDatabaseConnection, caoDatabaseDataFormat,
  caoDatabaseSQLCA, caoDatabaseParmMarkerFormat,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoDatabaseSQL exception object

  EcaoDatabaseSQL = class(Exception);

  TcaoSqlDataRetrievalMode = (
    caodbSynchronousRetrieval,
    caodbAsynchronousRetrieval);

  TcaoSqlDataToRetrieve = (
    caodbRetrieveData,
    caodbRetrieveDataFormat,
    caodbRetrieveSQLCA,
    caodbRetrieveParmMarkerFormat);

  TcaoDefaultDescribeOption = (
    caodbDescAliasNames,
    caodbDescNamesOnly,
    caodbDescLabels);

  TcaoDefaultScrollableCursor = (
    caodbCursorNotScrollable,
    caodbCursorScrollable);

  TcaoSqlDataToRetrieveSet = set of TcaoSqlDataToRetrieve;

  TcaoDatabaseSQL = class(TComponent)
  private
    { Private declarations }
    FReturnCode: integer;
    FActive: Boolean;
    FSetupCalled: boolean;
    FdataRetrievalMode: TcaoSqlDataRetrievalMode;
    FdataToRetrieve: TcaoSqlDataToRetrieveSet;
    FdefaultBlockCount: Integer;
    FdefaultDescribeOption: TcaoDefaultDescribeOption;
    FdefaultScrollableCursor: TcaoDefaultScrollableCursor;
    FdefaultStreamFetchSyncCount: Smallint;
    FcwbDatabaseSQL: cwbDatabaseSQL;
    FcaoErrorMessage: TcaoErrorMessage;
    FcaoDatabaseConnection: TcaoDatabaseConnection;
    FcaoDatabaseDataFormat: TcaoDatabaseDataFormat;
    FcaoDatabaseSQLCA: TcaoDatabaseSQLCA;
    FcaoDatabaseParmMarkerFormat: TcaoDatabaseParmMarkerFormat;
    FOnActive: TNotifyEvent;
    FOnCursorOpen: TNotifyEvent;
    FOnCursorClose: TNotifyEvent;
    function GetDataRetrievalMode: TcaoSqlDataRetrievalMode;
    procedure SetDataRetrievalMode(RetMode: TcaoSqlDataRetrievalMode);
    function GetDataToRetrieve: TcaoSqlDataToRetrieveSet;
    procedure SetDataToRetrieve(DataRet: TcaoSqlDataToRetrieveSet);
    function GetDefaultBlockCount: integer;
    procedure SetDefaultBlockCount(BlkCnt: integer);
    function GetResultDataLength: integer;
    function GetDefaultDescribeOption: TcaoDefaultDescribeOption;
    procedure SetDefaultDescribeOption(DesOpt: TcaoDefaultDescribeOption);
    function GetDefaultScrollableCursor: TcaoDefaultScrollableCursor;
    procedure SetDefaultScrollableCursor(ScrCur: TcaoDefaultScrollableCursor);
    function GetResultIndicatorCount: integer;
    function GetDefaultStreamFetchSyncCount: SmallInt;
    procedure SetDefaultStreamFetchSyncCount(SyncCnt: SmallInt);
    function GetActive: Boolean;
    procedure SetActive(Value: Boolean);
  protected
    { Protected declarations }
    function ResultCodeText: string; virtual;
    function GetErrMsg: string; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Setup; virtual;
    procedure GetHostErrInfo(var errorClass: Smallint; var errorCode: Integer; var messageID: WideString; var message1: WideString; var message2: WideString); virtual;
    procedure ChangeConnection(connection: IDispatch); virtual;
    procedure ClearPackage(const libraryName, packageName: WideString); virtual;
    procedure Close(const cursorName: WideString); virtual;
    procedure Commit(holdIndicator: Smallint); virtual;
    procedure Connect(const statementText: WideString); virtual;
    procedure CreatePackage(const libraryName, packageName: WideString); virtual;
    procedure DeletePackage(const libraryName, packageName: WideString); virtual;
    procedure Describe(const statementName: WideString); virtual;
    procedure DescribeParameterMarkers(const statementName: WideString); virtual;
    procedure DynamicStreamFetch(const statementText: WideString; var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer); virtual;
    procedure EndStreamFetch; virtual;
    procedure Execute(const statementName: WideString); virtual;
    procedure ExecuteImmediate(const statementName, statementText, cursorName: WideString); virtual;
    procedure ExtDynamicStreamFetch(const libName, pkgName, stmtName: WideString; var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer); virtual;
    procedure Fetch(const cursorName: WideString); virtual;
    procedure GetResultData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer); virtual;
    procedure MoreStreamData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer); virtual;
    procedure Open(const statementName, cursorName: WideString; openOptions: Smallint); virtual;
    procedure OpenDescribeFetch(const statementName, cursorName: WideString; openOptions: Smallint); virtual;
    procedure Prepare(const statementName, statementText: WideString); virtual;
    procedure PrepareDescribe(const statementName, statementText: WideString); virtual;
    procedure PrepareDescribeOpenFetch(const statementName, statementText, cursorName: WideString; openOptions: Smallint); virtual;
    procedure Rollback(holdIndicator: Smallint); virtual;
    procedure SetDefaultFetchScrollOpts(scrollType: Smallint; relativeDistance: Integer); virtual;
    procedure SetParameterMarkerBlock(numberOfRows: Integer; dataPointer, indicators: OleVariant); virtual;
    procedure SetParameterMarkers(dataBuf, indicators: OleVariant); virtual;
    procedure SetSQLCA; virtual;
    procedure SetParmMarkerFormat; virtual;
    procedure GetResultSQLCA; virtual;
    procedure GetResultDataFormat; virtual;
    procedure GetResultParmMarkerFormat; virtual;
    procedure SetDefaultPackageInfo(const libraryName, packageName: WideString); virtual;
    procedure GetDefaultPackageInfo(var libraryName: WideString; var packageName: WideString); virtual;
    procedure GetDefaultFetchScrollOpts(var scrollType: Smallint; var relativeDistance: Integer); virtual;
  published
    { Published declarations }
    property ReturnCode: integer read FReturnCode default cwbOK;
    property Active: Boolean read GetActive write SetActive default false;
    property dataRetrievalMode: TcaoSqlDataRetrievalMode read GetDataRetrievalMode write SetDataRetrievalMode default caodbSynchronousRetrieval;
    property dataToRetrieve: TcaoSqlDataToRetrieveSet read GetDataToRetrieve write SetDataToRetrieve default [caodbRetrieveData,caodbRetrieveDataFormat];
    property defaultBlockCount: integer read GetDefaultBlockCount write SetDefaultBlockCount default 0;
    property resultDataLength: integer read GetResultDataLength;
    property defaultDescribeOption: TcaoDefaultDescribeOption read GetDefaultDescribeOption write SetDefaultDescribeOption default caodbDescAliasNames;
    property defaultScrollableCursor: TcaoDefaultScrollableCursor read GetDefaultScrollableCursor write SetDefaultScrollableCursor default caodbCursorNotScrollable;
    property resultIndicatorCount: integer read GetResultIndicatorCount;
    property defaultStreamFetchSyncCount: Smallint read GetDefaultStreamFetchSyncCount write SetDefaultStreamFetchSyncCount default 0;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property databaseConnection: TcaoDatabaseConnection read FcaoDatabaseConnection write FcaoDatabaseConnection;
    property databaseDataFormat: TcaoDatabaseDataFormat read FcaoDatabaseDataFormat write FcaoDatabaseDataFormat;
    property databaseSQLCA: TcaoDatabaseSQLCA read FcaoDatabaseSQLCA write FcaoDatabaseSQLCA;
    property databaseParmMarkerFormat: TcaoDatabaseParmMarkerFormat read FcaoDatabaseParmMarkerFormat write FcaoDatabaseParmMarkerFormat;
    property OleObject: cwbDatabaseSQL read FcwbDatabaseSQL;
    property OnActive: TNotifyEvent read FOnActive write FOnActive;
    property OnCursorOpen: TNotifyEvent read FOnCursorOpen write FOnCursorOpen;
    property OnCursorClose: TNotifyEvent read FOnCursorClose write FOnCursorClose;
  end;

implementation

{
Create OLE object and initilize properties.
}

constructor TcaoDatabaseSQL.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbDatabaseSQL := CocwbDatabaseSQL.Create;
  FActive := false;
  FSetupCalled := false;
  FdataRetrievalMode := caodbSynchronousRetrieval;
  FdataToRetrieve := [caodbRetrieveData,caodbRetrieveDataFormat,
  caodbRetrieveSQLCA,caodbRetrieveParmMarkerFormat];
  FdefaultBlockCount := 0;
  FdefaultDescribeOption := caodbDescAliasNames;
  FdefaultScrollableCursor:= caodbCursorNotScrollable;
  FdefaultStreamFetchSyncCount := 2
end;

{
Close cwbDatabaseSQL.
}

destructor TcaoDatabaseSQL.Destroy;
begin
  if FActive then
    SetActive(false);
  inherited Destroy
end;

{
Return formatted result code text.
}

function TcaoDatabaseSQL.ResultCodeText: string;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoDatabaseSQL.GetErrMsg: string;
var
  S: string;
  errorClass: Smallint;
  errorCode: Integer;
  messageID: WideString;
  message1: WideString;
  message2: WideString;
  CauseStr: string;
  RecoveryStr: string;
  RecoveryPos: integer;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S <> '' then
  begin
    GetHostErrInfo(errorClass,errorCode,messageID,message1,message2);
    RecoveryPos := Pos('Recovery',message2);
    if RecoveryPos > 0 then
    begin
      CauseStr := Copy(message2,1,RecoveryPos-1);
      RecoveryStr := Copy(message2,RecoveryPos,Length(message2)-RecoveryPos)
    end
    else
    begin
      CauseStr := message2;
      RecoveryStr := '';
    end;
    S := Format('%s class: %d, code: %d, message ID: %s, %s'#10#10'%s'#10#10'%s',
    [S,errorClass,errorCode,messageID,message1,CauseStr,RecoveryStr])
  end
  else
    S := ResultCodeText;
  Result := S
end;

{
This property specifies when the result data set should be retrieved from the
AS/400.  Possible values are: cwbdbSynchronousRetrieval and
cwbdbAsynchronousRetrieval.  If the value is cwbdbSynchronousRetrieval, the
result data will be retrieved immediately, when a request is made for it.  If
the value is cwbdbAsynchronousRetrieval, the result data will not be retrieved
until an attempt is made to access it, by calling one of the cwbdbGetResult*
methods or by retrieving one of the result* properties.  When an attempt is
made to access one part of the result data, all result data for that request
will be retrieved.  If this property is not set, cwbdbSynchronousRetrieval is
used.  This is a read/write property.

Get OLE property dataRetrievalMode if setup was called.  Otherwise, get private
variable.
}

function TcaoDatabaseSQL.GetDataRetrievalMode: TcaoSqlDataRetrievalMode;
begin
  if FSetupCalled then
    case FcwbDatabaseSQL.defaultDescribeOption of
      cwbdbSynchronousRetrieval:
        Result := caodbSynchronousRetrieval;
      cwbdbAsynchronousRetrieval:
        Result := caodbAsynchronousRetrieval
      else
        Result := caodbSynchronousRetrieval
    end
  else
    Result := FdataRetrievalMode
end;

{
Set OLE property dataRetrievalMode if setup was called. Otherwise, set private
variable.
}

procedure TcaoDatabaseSQL.SetDataRetrievalMode(RetMode: TcaoSqlDataRetrievalMode);
begin
  if FSetupCalled then
    case RetMode of
      caodbSynchronousRetrieval:
        FcwbDatabaseSQL.dataRetrievalMode := cwbdbSynchronousRetrieval;
      caodbAsynchronousRetrieval:
        FcwbDatabaseSQL.dataRetrievalMode := cwbdbAsynchronousRetrieval
    end
  else
    FdataRetrievalMode := RetMode
end;

{
This property specifies which part(s) of the result set should be retrieved
from the AS/400.  This may any combination of the following bit constants ORed
together.

	For a cwbDatabaseCatalog object:

cwbdbRetrieveData - data and indicators
cwbdbRetrieveDataFormat - data format

	For a cwbDatabaseSQL object:

cwbdbRetrieveData - data and indicators
cwbdbRetrieveDataFormat - data format
cwbdbRetrieveSQLCA - SQLCA information
cwbdbRetrieveParmMarkerFormat - parameter marker format

If this property is not set, the entire result set is retrieved.  If a certain
type of data is specifed but, it is not part of the result set for a particular
request, no attempt is made to retrieve it.  This is a read/write property.

Get OLE property dataToRetrieve if setup was called.  Otherwise, get private
variable.
}

function TcaoDatabaseSQL.GetDataToRetrieve: TcaoSqlDataToRetrieveSet;
var
  DataRet: SmallInt;
begin
  if FSetupCalled then
  begin
    DataRet := FcwbDatabaseSQL.dataToRetrieve;
    Result := [];
    if (DataRet and Smallint(cwbdbRetrieveData)) =
    SmallInt(cwbdbRetrieveData) then
      Result := Result+[caodbRetrieveData];
    if (DataRet and Smallint(cwbdbRetrieveDataFormat)) =
    SmallInt(cwbdbRetrieveDataFormat) then
      Result := Result+[caodbRetrieveDataFormat];
    if (DataRet and Smallint(cwbdbRetrieveSQLCA)) =
    SmallInt(cwbdbRetrieveSQLCA) then
      Result := Result+[caodbRetrieveSQLCA];
    if (DataRet and Smallint(cwbdbRetrieveParmMarkerFormat)) =
    SmallInt(cwbdbRetrieveParmMarkerFormat) then
      Result := Result+[caodbRetrieveParmMarkerFormat]
  end
  else
    Result := FdataToRetrieve
end;

{
Set OLE property dataRetrievalMode if setup was called. Otherwise, set private
variable.
}

procedure TcaoDatabaseSQL.SetDataToRetrieve(DataRet: TcaoSqlDataToRetrieveSet);
var
  Value: SmallInt;
begin
  if FSetupCalled then
  begin
    Value := 0;
    if caodbRetrieveData in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveData);
    if caodbRetrieveDataFormat in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveDataFormat);
    if caodbRetrieveSQLCA in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveSQLCA);
    if caodbRetrieveParmMarkerFormat in DataRet then
      Value := Value or ShortInt(cwbdbRetrieveParmMarkerFormat);
    FcwbDatabaseSQL.dataToRetrieve := Value
  end
  else
    FdataToRetrieve := DataRet
end;

{
This property specifies how many rows should be blocked together when
transferring data between the client and the AS/400.  The value of this
property is used when any of the following methods are called:

cwbdbExecuteImmediate
cwbdbFetch
cwbdbOpen
cwbdbOpenDescribeFetch
cwbdbPrepareDescribeOpenFetch

If this property is not set, zero will be used.  This is a read/write property.

Get OLE property defaultBlockCount if setup was called.  Otherwise, get private
variable.
}

function TcaoDatabaseSQL.GetDefaultBlockCount: integer;
begin
  if FSetupCalled then
    Result := FcwbDatabaseSQL.defaultBlockCount
  else
    Result := FdefaultBlockCount
end;

{
Set OLE property defaultBlockCount if setup was called. Otherwise, set private
variable.
}

procedure TcaoDatabaseSQL.SetDefaultBlockCount(BlkCnt: integer);
begin
  if FSetupCalled then
    FcwbDatabaseSQL.defaultBlockCount := BlkCnt
  else
    FdefaultBlockCount := BlkCnt
end;

{
This property contains the size of the data portion of the result set, for the
most recent request. This property can be retrieved to to determine the size
of the data buffer needed for a cwbdbGetResultData call.  This is a read-only
property.
}

function TcaoDatabaseSQL.GetResultDataLength: integer;
begin
  Result := FcwbDatabaseSQL.resultDataLength
end;

{
This property specifies what values should be used, for the column names
stored in the cwbDatabaseDataFormat object, when the result set is described
by one of the following methods:

cwbdbDescribe
cwbdbPrepareDescribe
cwbdbExecuteImmediate
cwbdbOpenDescribeFetch

Possible values are:

cwbdbDescAliasNames

Uses the name of the column

cwbdbDescNamesOnly

Uses the system column name of the column

cwbdbDescLabels

Uses the label of the column

If this property is not set, cwbdbDescAliasNames is used.  This is a
read/write property.

Get OLE property defaultDescribeOption if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseSQL.GetDefaultDescribeOption: TcaoDefaultDescribeOption;
begin
  if FSetupCalled then
    case FcwbDatabaseSQL.defaultDescribeOption of
      cwbdbDescAliasNames:
        Result := caodbDescAliasNames;
      cwbdbDescNamesOnly:
        Result := caodbDescNamesOnly;
      cwbdbDescLabels:
        Result := caodbDescLabels
      else
        Result := caodbDescAliasNames
    end
  else
    Result := FdefaultDescribeOption
end;

{
Set OLE property defaultDescribeOption if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseSQL.SetDefaultDescribeOption(DesOpt: TcaoDefaultDescribeOption);
begin
  if FSetupCalled then
    case DesOpt of
      caodbDescAliasNames:
        FcwbDatabaseSQL.defaultDescribeOption := cwbdbDescAliasNames;
      caodbDescNamesOnly:
        FcwbDatabaseSQL.defaultDescribeOption := cwbdbDescNamesOnly;
      caodbDescLabels:
        FcwbDatabaseSQL.defaultDescribeOption := cwbdbDescLabels
    end
  else
    FdefaultDescribeOption := DesOpt
end;

{
This property indicates whether or not the cursor used when fetching data from
the AS/400 with a call to cwbdbExecuteImmediate, cwbdbOpen, or
cwbdbOpenDescribeFetch is scrollable.  Possible values are:
cwbdbCursorNotScrollable and cwbdbCursorScrollable.  If this property is not
set, cwbdbCursorNotScrollable is used.  This is a read/write property.

Get OLE property defaultScrollableCursor if setup was called.  Otherwise, get
private variable.
}

function TcaoDatabaseSQL.GetDefaultScrollableCursor: TcaoDefaultScrollableCursor;
begin
  if FSetupCalled then
    case FcwbDatabaseSQL.defaultScrollableCursor of
      cwbdbCursorNotScrollable:
        Result := caodbCursorNotScrollable;
      cwbdbCursorScrollable:
        Result := caodbCursorScrollable
      else
        Result := caodbCursorNotScrollable
    end
  else
    Result := FdefaultScrollableCursor
end;

{
Set OLE property defaultScrollableCursor if setup was called. Otherwise, set
private variable.
}

procedure TcaoDatabaseSQL.SetDefaultScrollableCursor(ScrCur: TcaoDefaultScrollableCursor);
begin
  if FSetupCalled then
    case ScrCur of
      caodbCursorNotScrollable:
        FcwbDatabaseSQL.defaultScrollableCursor := cwbdbCursorNotScrollable;
      caodbCursorScrollable:
        FcwbDatabaseSQL.defaultScrollableCursor := cwbdbCursorScrollable
    end
  else
    FdefaultScrollableCursor := ScrCur
end;

{
This property contains the number of the indicators for the result set of the
most recent request.  This property can be retrieved to to determine the size
of the indicator buffer needed for a cwbdbGetResultData call.  This is a
readonly property.
}

function TcaoDatabaseSQL.GetResultIndicatorCount: integer;
begin
  Result := FcwbDatabaseSQL.resultIndicatorCount
end;

{
Get OLE property defaultStreamFetchSyncCount if setup was called.  Otherwise,
get private variable.
}

function TcaoDatabaseSQL.GetDefaultStreamFetchSyncCount: SmallInt;
begin
  if FSetupCalled then
    Result := FcwbDatabaseSQL.defaultStreamFetchSyncCount
  else
    Result := FdefaultStreamFetchSyncCount
end;

{
Set OLE property defaultStreamFetchSyncCount if setup was called. Otherwise,
set private variable.
}

procedure TcaoDatabaseSQL.SetDefaultStreamFetchSyncCount(SyncCnt: SmallInt);
begin
  if FSetupCalled then
    FcwbDatabaseSQL.defaultStreamFetchSyncCount := SyncCnt
  else
    FdefaultStreamFetchSyncCount := SyncCnt
end;

{
Return Active value.
}

function TcaoDatabaseSQL.GetActive: Boolean;
begin
  Result := FActive
end;

{
Set Active value and call Setup based on value.
}

procedure TcaoDatabaseSQL.SetActive(Value: Boolean);
begin
  if not FSetupCalled then
    Setup;
  FActive := Value;
  if FActive then
    if Assigned(OnActive) then
      OnActive(Self)
end;

{
Handle caoErrorMessage, caoDatabaseConnection, caoDatabaseDataFormat,
caoDatabaseSQLCA or caoDatabaseParmMarkerFormat component being removed.
}

procedure TcaoDatabaseSQL.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FcaoErrorMessage then
      FcaoErrorMessage := nil
    else
      if AComponent = FcaoDatabaseConnection then
        FcaoDatabaseConnection := nil
      else
        if AComponent = FcaoDatabaseDataFormat then
          FcaoDatabaseDataFormat := nil
        else
          if AComponent = FcaoDatabaseSQLCA then
            FcaoDatabaseSQLCA := nil
          else
            if AComponent = FcaoDatabaseParmMarkerFormat then
              FcaoDatabaseParmMarkerFormat := nil
  end
end;

{
This method initializes the object.  It must be called before any other method
or before accessing any of the objects properties.

Parameters:
connection - Object
Identifies the cwbDatabaseConnection object to be used for servicing requests
of this object.
format - Object
Identifies the cwbDatabaseDataFormat object to be used for storing the format
of the result data for the most recent request.
}

procedure TcaoDatabaseSQL.Setup;
begin
  // Fire up database connection if not active
  if not FcaoDatabaseConnection.Active then
    FcaoDatabaseConnection.Active := true;
  FReturnCode := FcwbDatabaseSQL.cwbdbSetup(
  FcaoDatabaseConnection.OleObject,FcaoDatabaseDataFormat.OleObject,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(ResultCodeText);
  FSetupCalled := true;
  FActive := true;
  // Set OLE properties from variables
  SetDataRetrievalMode(FdataRetrievalMode);
  SetDataToRetrieve(FdataToRetrieve);
  SetDefaultBlockCount(FdefaultBlockCount);
  SetDefaultDescribeOption(FdefaultDescribeOption);
  SetDefaultScrollableCursor(FdefaultScrollableCursor);
  SetDefaultStreamFetchSyncCount(FdefaultStreamFetchSyncCount)
end;

{
This method returns any host error information associated with the most recent
request.

Parameters:
errorClass - Pointer to Short Integer
Specifies a buffer where the error class will be stored.  This class indicates
which database server module encountered an error.  Possible values are:

0 - no error
1 - SQL functional error
2 - SQL parameter error
3 - List functional error
4 - List parameter error
5 - DB2 functional error
6 - DB2 parameter error
7 - General server error
8 - User exit error

errorCode - Pointer to Long
Specifies a buffer where the return code from the server module will be stored.
messageID - Pointer to String
Provides a location to store the host message identifier.  (This should be
passed as a pointer to a BSTR.)
message1 - Pointer to String
Provides a location to store the first host message.  (This should be passed
as a pointer to a BSTR.)
message2 - Pointer to String
Provides a location to store the second host message identifier.  (This should
be passed as a pointer to a BSTR.)
}

procedure TcaoDatabaseSQL.GetHostErrInfo(var errorClass: Smallint; var errorCode: Integer; var messageID: WideString; var message1: WideString; var message2: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetHostErrInfo(errorClass,errorCode,
  messageID,message1,message2,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method provides a way to switch which cwbDatabaseConnection object is used
for the catalog requests made by this object.

Parameters:
connection - Object
Identifies the new cwbDatabaseConnection object to be used for servicing
requests of this object.
}

procedure TcaoDatabaseSQL.ChangeConnection(connection: IDispatch);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbChangeConnection(
  connection,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method clears all statements from a SQL package.

Parameters:
libraryName - String
Identifies the library in which the package exists.
packageName - String
Identifes the package to clear.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.ClearPackage(const libraryName, packageName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbClearPackage(
  libraryName,packageName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method closes an open cursor.

Parameters:
cursorName - String
Specifies the name of the cursor to close.  This must be the same name used on
the request which opened the cursor.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Close(const cursorName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbClose(
  cursorName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg);
  if Assigned(OnCursorClose) then
    OnCursorClose(Self)
end;

{
This method commits a unit of work to the database.  After the unit of work has
been committed, it can no longer be undone by a call to the cwbdbRollback
method.  See the commitmentControl property of the cwbDatabaseConnection object
for information on how to set the dafault commitment level.

Parameters:
holdIndicator - Short Integer
Specifies whether or not the cursor should remain open after the commit
operation.  Possible values are cwbdbWork (close the cursor) and cwbdbHold
(keep the cursor open).

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Commit(holdIndicator: Smallint);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbCommit(
  holdIndicator,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method performs a Distributed Relational Database Architecture (DRDA)
connection management function.  It is used to establish and switch between
connections to other Relational Databases.

Parameters:
statementText - String
Conatins the connect statement specifying which system to connect to.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Connect(const statementText: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbConnect(
  statementText,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method creates a SQL package into which statements can be prepared.  Once
a statement has been prepared into a package, it can be reused without having
to prepare it again.

Parameters:
libraryName - String
Identifies the library in which the package should be created.
packageName - String
Identifes the package to create.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.CreatePackage(const libraryName, packageName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbCreatePackage(
  libraryName,packageName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method deletes a SQL package.

Parameters:
libraryName - String
Identifies the library in which the package exists.
packageName - String
Identifies the package to delete.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.DeletePackage(const libraryName, packageName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbDeletePackage(
  libraryName,packageName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method determines the data types of the result set for a prepared statement.  If there is no result set, no column descriptions will be returned.

Parameters:
statementName - String
Specifies the name of the request to be described.  This should be the same name used when the statement was prepared.

Before this method is called, the following property(ies) may be set:

defaultDescribeOption

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
package information will be used to locate the statement.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Describe(const statementName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbDescribe(
  statementName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method determines the data types of the parameter markers for a prepared
statement.  If the statement is an "UPDATE WHERE CURRENT OF CURSOR", the cursor
must be open before the describe parameter markers can be performed.  Note:
The cwbdbSetParmMarkerFormat method must be called before calling this method.

Parameters:
statementName - String
Specifies the name of the request to be described.  This should be the same
name used when the statement was prepared.

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
package information will be used to locate the statement.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultParmMarkerFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.DescribeParameterMarkers(const statementName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbDescribeParameterMarkers(
  statementName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method prepares a select statement, opens a cursor and fetchs all
resulting data. The row data will be returned to the application in blocks, the
size of which will be optimized for the communication mechanism.  To get
additional blocks, use the cwbdbMoreStreamData method.  To cancel the stream
fetch operation, before all the data has been returned, use the
cwbdbEndStreamFetch method.  If the record length of the file is close to 32 K,
the maximum block size will be 64K.  Otherwise, the maximum block size will be
32K.

Parameters:
statementText - String
Identifes the select statement to be issued.
dataBuf - Byte Array Variant
Specifies a buffer to be used to store the result data.  (This buffer should be
passed as a SafeArray of unsigned characters.)
dataLen - Pointer to Long
Used to store the number of indicators stored in dataBuf, on output. If the
Return Code is cwbBufferOverflow, this parameter will contain the size of the
buffer needed to staisfy this request.

indicatorBuf - Short Integer Array Variant
Specifies a buffer to be used to store the result indicators.  (This buffer
should be passed as a SafeArray of signed short integers.)
indicatorCount - Pointer to Long
Used to store the number of indicators stored in indicatorBuf, on output.  If
the Return Code is cwbBufferOverflow, this parameter will contain the number
of indicators in the result set for this request.

Before this method is called, the following properties may be set:

defaultStreamFetchSyncCount
}

procedure TcaoDatabaseSQL.DynamicStreamFetch(const statementText: WideString; var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbDynamicStreamFetch(
  statementText,dataBuf,dataLen,indicatorBuf,indicatorCount,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method cancels a cwbdbDynamicStreamFetch operation before all the data has
been returned.
}

procedure TcaoDatabaseSQL.EndStreamFetch;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbEndStreamFetch(
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method executes a prepared SQL statement.

Parameters:
statementName - String
Identifes the statement to be executed.  This should be the same name used when
the statement was prepared.

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
package information will be used to locate the statement.

If parameter markers will be used for this request,
cwbdbSetParameterMarkerBlock or cwbdbSetParameterMarkers must be called before
this method.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Execute(const statementName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbExecute(
  statementName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method prepares and executes a SQL statement.

Parameters:
statementName - String
Specifies a name to be used to identify this particular request.
statementText - String
Identifes the statement to be issued.
cursorName - String
Specifies a name to be used to identify the current access position of this
request.

Before this method is called, the following properties may be set:

defaultBlockCount
defaultScrollableCursor
defaultDescribeOption

If parameter markers will be used for this request,
cwbdbSetParameterMarkerBlock or cwbdbSetParameterMarkers must be called before
this method.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.ExecuteImmediate(const statementName, statementText, cursorName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbExecuteImmediate(
  statementName,statementText,cursorName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method opens a cursor and fetchs all resulting data for a select statement
that has already been prepared in a SQL package. The row data will be returned
to the application in blocks, the size of which will be optimized for the
communication mechanism.  To get additional blocks, use cwbdbMoreStreamData
method.  To cancel the stream fetch operation before all the data has been
returned, use the cwbdbEndStreamFetch method.  If the record length of the file
is close to 32K, the maximum block size will be 64K.  Otherwise, the maximum
block size will be 32K.

Parameters:
libName - String
Identifies the library in which the package exists.
pkgName - String
Identifes the package containing the prepared statement.
stmtName - String
Identifes the select statement within the package to be executed.  This should
be the same name used when the statement was prepared.
dataBuf - Byte Array Variant
Specifies a buffer to be used to store the result data.  (This buffer should be
passed as a SafeArray of unsigned characters.)

dataLen - Pointer to Long
Used to store the number of bytes stored in dataBuf, on output.  If the return
code is cwbBufferOverflow, this parameter will contain the size of the buffer
needed to satisfy this request.
indicatorBuf - Short Integer Array Variant
Specifies a buffer to be used to store the result indicators.  (This buffer
should be passed as a SafeArray of signed short integers.)
indicatorCount - Pointer to Long
Used to store the number of indicators stored in indicatorBuf, on output.  If
the Return Code is cwbBufferOverflow, this parameter will contain the number
of indicators in the result set for this request.

Before this method is called, the following properties may be set:

defaultStreamFetchSyncCount
}

procedure TcaoDatabaseSQL.ExtDynamicStreamFetch(const libName, pkgName, stmtName: WideString; var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbExtDynamicStreamFetch(
  libName,pkgName,stmtName,dataBuf,dataLen,indicatorBuf,indicatorCount,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method fetchs a row or block of rows from an open cursor.

Parameters:
cursorName - String
Specifies the name used to identify the current access position of this
request. This should be the same name used when the cursor was opened.

Before this method is called, the following property may be set:

defaultBlockCount

If the value of the defaultScrollableCursor property is cwbdbCursorScrollable,
the cwbdbSetDefaultFetchScrollOpts method may be called, before this one, to
specify how to scroll through the data.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Fetch(const cursorName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbFetch(
  cursorName,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method returns the data portion of the result set for the most recent
requesst.  If the dataToRetrieve property value does not include
cwbdbRetrieveData, no data will be returned.

Parameters:
dataBuf - Byte Array Variant
Specifies a buffer to be used to store the data.  (This buffer should be passed
as a SafeArray of unsigned characters.)
dataLen - Pointer to Long
Used to store the number of bytes stored in dataBuf, on output.  If the Return
Code is cwbBufferOverflow, this parameter will contain the size of the buffer
needed to satisfy this request.  The resultDataLength may also be retrieved,
before calling this method, to determine the buffer size.

indicatorBuf - Short Integer Array Variant
Specifies a buffer to be used to store the null data indicators.  Null
indicators are only returned if at least one of the columns in the result set
is null capable.  There is one indicator returned for each column of each row
in the result set.  Each entry specifes if the corresponding data value is null
(-1) or if it experienced a conversion error ('E' or 0x0045).  Otherwise, the
value is zero.  The indicators for the first row are stored in the array first,
followed by the indicators for the second row, and so on.  If the indicators
are not wanted, this value should be NULL.  (This should be passed as a
SafeArray of signed short integers.)

indicatorCount - Pointer to Long
Used to store the number of null indicators stored in indicatorBuf, on output.
If the Return Code is cwbBufferOverflow, this parameter will contain the number
of indicators in the result set.  The resultIndicatorCount property value may
also be retrieved, before calling this method, to determine the number of
indicators.
}

procedure TcaoDatabaseSQL.GetResultData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetResultData(
  dataBuf,dataLen,indicatorBuf,indicatorCount,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method retrieves the next block of stream fetch data resulting from a
cwbdbDynamicStreamFetch or cwbdbExtDynamicStreamFetch request.  If the record
length of the file is close to 32K, the maximum block size will be 64K.
Otherwise, the maximum block size will be 32K.

Parameters:
dataBuf - Byte Array Variant
Specifies a buffer to be used to store the result data.  (This buffer should be
passed as a SafeArray of unsigned characters.)

dataLen - Pointer to Long
Used to store the number of bytes stored in dataBuf, on output.  If the return
code is cwbBufferOverflow, this parameter will contain the size of the buffer
needed to satisfy this request.
indicatorBuf - Short Integer Array Variant
Specifies a buffer to be used to store the result indicators.  (This buffer
should be passed as a SafeArray of signed short integers.)
indicatorCount - Pointer to Long
Used to store the number of indicators stored in indicatorBuf, on output.  If
the Return Code is cwbBufferOverflow, this parameter will contain the number of
indicators in the result set for this request.
}

procedure TcaoDatabaseSQL.MoreStreamData(var dataBuf: OleVariant; var dataLen: Integer; var indicatorBuf: OleVariant; var indicatorCount: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbMoreStreamData(
  dataBuf,dataLen,indicatorBuf,indicatorCount,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method opens the database file and initializes the cursor position.

Parameters:
statementName - String
Specifies the name of the request for which the cursor is being opened.  This
should be the same name used when the statement was prepared.
cursorName - String
Specifies a name to be used to identify the current access position of this
request.
openOptions - Short Integer
Identifes the access intent of the open cursor.  Possible values are:

cwbdbRead
cwbdbWrite
cwbdbUpdate
cwbdbDelete
cwbdbOpenAll

Before this method is called, the following property(ies) may be set:
defaultDescribeOption, defaultScrollableCursor, or defaultBlockCount.

If package information was set by a call to cwbdbSetDefaultPackageInfo,
the package information will be used to locate the statement.

If parameter markers will be used for this request,
cwbdbSetParameterMarkerBlock or cwbdbSetParameterMarkers must be called before
this method.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Open(const statementName, cursorName: WideString; openOptions: Smallint);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbOpen(
  statementName,cursorName,openOptions,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg);
  if Assigned(OnCursorOpen) then
    OnCursorOpen(Self)
end;

{
This method opens a cursor, determines the data types of the result set for a
prepared statement, and fetchs a row or block of rows from the open cursor.
This combined function is valuable when the statement is already prepared
(extended dynamic SQL).

Parameters:
statementName - String
Specifies the name of the prepared statement.  This should be the same name
used when the statement was prepared.
cursorName - String
Specifies a name to be used to identify the current access position of this
request.

openOptions - Short Integer
Indentifes the access intent of the open cursor.  Possible values are:
cwbdbRead, cwbdbWrite, cwbdbUpdate, cwbdbDelete, and cwbdbOpenAll.

Before this method is called, the following properties may be set:

defaultBlockCount
defaultScrollableCursor
defaultDescribeOption

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
package information will be used to locate the statement.
If the value of the defaultScrollableCursor property is cwbdbCursorScrollable,
the cwbdbSetDefaultFetchScrollOpts method may be called, before this one, to
specify how to scroll through the data.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.OpenDescribeFetch(const statementName, cursorName: WideString; openOptions: Smallint);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbOpenDescribeFetch(
  statementName,cursorName,openOptions,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg);
  if Assigned(OnCursorOpen) then
    OnCursorOpen(Self)
end;

{
This method performs syntax checking and other general setup for a SQL
statement.

Parameters:
statementName - String
Specifies a name to be used to identify this particular request.
statementText - String
Identifes the text of the statement to be prepared.

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
statement will be prepared into the package.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.Prepare(const statementName, statementText: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbPrepare(
  statementName, statementText,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method performs general setup and determines the result set data types for
a SQL statement.  The advantage of using this method in place of separate calls
to cwbdbPrepare and cwbdbDescribe is that the SQL component is called only once.

Parameters:
statementName - String
Specifies a name to be used to identify this particular request.
statementText - String
Identifes the text of the statement.

Before this method is called, the following property(ies) may be set:

defaultDescribeOption

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
statement will be prepared into the package.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.PrepareDescribe(const statementName, statementText: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbPrepareDescribe(
  statementName,statementText,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method prepares and describes a SQL statement, opens a cursor, and fetchs
the data.  Combining these operations has a performance advantage over
individual calls to cwbdbPrepare, cwbdbDescribe, cwbdbOpen, and cwbdbFetch,
since only one call is made to the SQL component on the host.

Parameters:
statementName - String
Specifies a name to be used to identify this particular request.
statementText - String

Identifes the statement to be issued.
cursorName - String
Specifies a name to be used to identify the current access position  of this
request.
openOptions - Short Integer
Identifes the access intent of the open cursor.  Possible values are:
cwbdbRead, cwbdbWrite, cwbdbUpdate, cwbdbDelete, and cwbdbOpenAll.

Before this method is called, the following property(ies) may be set:

defaultDescribeOption
defaultBlockCount

If package information was set by a call to cwbdbSetDefaultPackageInfo, the
statement will be prepared into the package.

If parameter markers will be used for this request,
cwbdbSetParameterMarkerBlock or cwbdbSetParameterMarkers must be called before
this method.

After this method is called, the following methods may be called to obtain the
results of the request:

cwbdbGetResultData
cwbdbGetResultDataFormat
cwbdbGetResultSQLCA
cwbdbGetHostErrInfo
}

procedure TcaoDatabaseSQL.PrepareDescribeOpenFetch(const statementName, statementText, cursorName: WideString; openOptions: Smallint);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbPrepareDescribeOpenFetch(
  statementName,statementText,cursorName,openOptions,
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method rolls back a unit of work which has not been committed by a call to
the cwbdbCommit method or by the default commitment behavior defined by the
commitmentControl property of the cwbDatabaseConnection object.

Parameters:
holdIndicator - Short Integer
Specifies whether or not the cursor should remain open after the rollback
operation.  Possible values are cwbdbWork (close the cursor) and cwbdbHold
(keep the cursor open).
}

procedure TcaoDatabaseSQL.Rollback(holdIndicator: Smallint);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbRollback(
  holdIndicator,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method retrieves information about how the cursor will scroll through the
data.  This information can be set by calling cwbdbSetDefaultFetchScrollOpts.

Parameters:
ScrollType - Pointer to Short Integer
Identifies a buffer used to store the type of scrolling to be performed.
Possible values are:

cwbdbScrollNext
cwbdbScrollPrevious
cwbdbScrollFirst
cwbdbScrollLast
cwbdbScrollBeforeFirst
cwbdbScrollAfterLast
cwbdbScrollCurrent
cwbdbScrollRelative

relativeDistance - Pointer to Long
Identifies a buffer used to store the relative distance from the current cursor
position to scroll, if the scroll type is cwbdbScrollRelative.
}

procedure TcaoDatabaseSQL.SetDefaultFetchScrollOpts(scrollType: Smallint; relativeDistance: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetDefaultFetchScrollOpts(
  scrollType,relativeDistance);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method provides the data, for a block of rows, to be used for the
parameter markers contained in a prepared statement.

Parameters:
numberOfRows - Long
Specifies the number of rows for which parameter marker data is supplied.
dataPointer - Byte Array Variant
Identifes the buffer containing the the data for the parameter markers.  (This
buffer should be passed as a SafeArray of unsigned characters.)
indicators - Short Integer Array Variant

Identifies the buffer containing the null indicators.  If the value of an
indicator is less than zero, the value for the corresponding parameter marker
is null. (This buffer should be passed as a SafeArray of signed short integers.)
}

procedure TcaoDatabaseSQL.SetParameterMarkerBlock(numberOfRows: Integer; dataPointer, indicators: OleVariant);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetParameterMarkerBlock(
  numberOfRows,dataPointer,indicators,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method provides the data, for one row, to be used for the parameter
markers contained in a prepared statement.

Parameters:
dataBuf - Byte Array Variant
Identifes the buffer containing the the data for the parameter markers.  (This
buffer should be passed as a SafeArray of unsigned characters.)
indicators - Short Integer Array Variant
Identifies the buffer containing the null indicators.  If the value of an
indicator is less than zero, the value for the corresponding parameter marker
is null. (This buffer should be passed as a SafeArray of signed short integers.)
}

procedure TcaoDatabaseSQL.SetParameterMarkers(dataBuf, indicators: OleVariant);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetParameterMarkers(
  dataBuf,indicators,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method supplies a cwbDatabaseSQLCA object to be used for storing the state
of the SQL statement associated with the most recent request.  After a SQL
request is made, the cwbdbGetResultSQLCA method can be called to store the SQL
state information into the object supplied.
}

procedure TcaoDatabaseSQL.SetSQLCA;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetSQLCA(
  FcaoDatabaseSQLCA.OleObject,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method supplies a cwbDatabaseParmMarkerFormat object to be used for
storing the format of the parameter markers associated with a SQL request.
After a cwbdbDescribeParameterMarkers request is made, the
cwbdbGetResultParmMarkerFormat method can be called to store the parameter
marker format information into the object supplied.
}

procedure TcaoDatabaseSQL.SetParmMarkerFormat;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetParmMarkerFormat(
  FcaoDatabaseParmMarkerFormat.OleObject,FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method puts information about the state SQL statement, for the most recent
request, into the cwbDatabaseSQLCA object specified on the cwbdbSetSQLCA method
call.  After this method is called, the methods of the cwbDatabaseSQLCA object
can be used to access the state information.  If the cwbdbSetSQLCA method was
not called or the dataToRetrieve property value does not include
cwbdbRetrieveSQLCA, this method does nothing.
}

procedure TcaoDatabaseSQL.GetResultSQLCA;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetResultSQLCA(
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method puts the data format information of the result set, for the most
recent request, into the cwbDatabaseDataFormat object specified on the
cwbdbSetup method call.  After this method is called, the methods of the
cwbDatabaseDataFormat object can be used to access the data format information.
If the dataToRetrieve property value does not include cwbdbRetrieveDataFormat,
this method does nothing.
}

procedure TcaoDatabaseSQL.GetResultDataFormat;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetResultDataFormat(
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method puts the parameter marker format information, for the most recent
request, into the cwbDatabaseParmMarkerFormat object specified on the
cwbdbSetParmMarkerFormat method call.  After this method is called, the methods
of the cwbDatabaseParmMarkerFormat object can be used to access the parameter
marker format information.  If the cwbdbSetParmMarkerFormat method was not
called or the dataToRetrieve property value does not include
cwbdbRetrieveParmMarkerFormat, this method does nothing.
}

procedure TcaoDatabaseSQL.GetResultParmMarkerFormat;
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetResultParmMarkerFormat(
  FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method specifies the name of the SQL package and the library containing
it, to be used for preparing or executing SQL statements.  If these strings are
not null, prepare requests will prepare the statement into the specified
package.  Also, when a request is made to execute an SQL statement, a search
will be made for the statement in the package.  If this method is not called or
if these values are set to null strings, a package will not be used.

Parameters:
libraryName - String

Identifies the library in which the package exists.
packageName - String
Identifies the package to use.
}

procedure TcaoDatabaseSQL.SetDefaultPackageInfo(const libraryName, packageName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbSetDefaultPackageInfo(
  libraryName,packageName);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method retrieves the name of the SQL package and the name of the library
containing it.  These values are used for preparing and executing statements
using a package.  If these strings are null, a package will not be used.  The
cwbdbSetDefaultPackageInfo method can be called to set these values.

Parameters:
libraryName - Pointer to String
Provides a location to store the library name containing the package to use.
(This should be passed as a pointer to a BSTR.)

packageName - Pointer to String
Provides a location to store the package name to use.  (This should be passed as
a pointer to a BSTR.)
}

procedure TcaoDatabaseSQL.GetDefaultPackageInfo(var libraryName: WideString; var packageName: WideString);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetDefaultPackageInfo(
  libraryName,packageName);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

{
This method retrieves information about how the cursor will scroll through the
data.  This information can be set by calling cwbdbSetDefaultFetchScrollOpts.

Parameters:
ScrollType - Pointer to Short Integer
Identifies a buffer used to store the type of scrolling to be performed.
Possible values are:

cwbdbScrollNext
cwbdbScrollPrevious
cwbdbScrollFirst
cwbdbScrollLast
cwbdbScrollBeforeFirst
cwbdbScrollAfterLast
cwbdbScrollCurrent
cwbdbScrollRelative

relativeDistance - Pointer to Long
Identifies a buffer used to store the relative distance from the current cursor
position to scroll, if the scroll type is cwbdbScrollRelative.
}

procedure TcaoDatabaseSQL.GetDefaultFetchScrollOpts(var scrollType: Smallint; var relativeDistance: Integer);
begin
  FReturnCode := FcwbDatabaseSQL.cwbdbGetDefaultFetchScrollOpts(
  scrollType,relativeDistance);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoDatabaseSQL.Create(GetErrMsg)
end;

end.
