{
Client Access Objects 1.5

caoErrorMessage component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbErrorMessage automation object holds error messages associated with a
particular request.  A cwbErrorMessage object can be used on any request which
takes an object of its type as a parameter.  If the request fails, the
properties and methods of this object can be used to determine what went wrong.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoErrorMessage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoErrorMessage exception object

  EcaoErrorMessage = class(Exception);

  // TcaoErrorMessage component

  TcaoErrorMessage = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Integer;
    FcwbErrorMessage: cwbErrorMessage;
    function GetErrorCount: Integer;
  protected
    { Protected declarations }
    function ResultCodeText: string; virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Setup; virtual;
    function GetErrClassIndexed(errorIndex: Integer): Integer; virtual;
    function GetErrTextIndexed(errorIndex: Integer): WideString; virtual;
    function GetErrMsg: string; virtual;
  published
    { Published declarations }
    property ReturnCode: Integer read FReturnCode default cwbOK;
    property errorCount: Integer read GetErrorCount;
    property OleObject: cwbErrorMessage read FcwbErrorMessage;
  end;

implementation

{
Create OLE object.
}

constructor TcaoErrorMessage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FcwbErrorMessage := CocwbErrorMessage.Create
end;

{
Return formatted result code text.
}

function TcaoErrorMessage.ResultCodeText: string;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
This method initializes the error message object.  It must be called before any
other method or before accessing any of the objects properties.
}

procedure TcaoErrorMessage.Setup;
begin
  FReturnCode := FcwbErrorMessage.cwbsvSetup;
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoErrorMessage.Create(ResultCodeText)
end;

{
This property contains the number of messages currently associated with the
error message object.  This is a read-only property.
}

function TcaoErrorMessage.GetErrorCount: Integer;
begin
  Result := FcwbErrorMessage.errorCount
end;

{
This method returns the message class associated with the error index provided.
An index value of 1 retrieves the lowest-level or oldest message associated
with the error message object.  Using an index equal to the errorCount property
value retrieves the top-level or most recent message associated with the error
message object.

Possible values for the error class are:

cwbsvClassInformational
cwbsvClassErrorRetry
cwbsvClassErrorAbort
}

function TcaoErrorMessage.GetErrClassIndexed(errorIndex: Integer): Integer;
var
  errorClass: Integer;
begin
  FReturnCode := FcwbErrorMessage.cwbsvGetErrClassIndexed(errorIndex,errorClass);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoErrorMessage.Create(ResultCodeText);
  Result := errorClass
end;

{
This method returns the message text associated with the index provided.  An
index of one retrieves the lowest level or oldest message associated with the
error message object.  Using an index equal to the errorCount property value
retrieves the top-level or most recent message.
}

function TcaoErrorMessage.GetErrTextIndexed(errorIndex: Integer): WideString;
var
  errorText: WideString;
begin
  FReturnCode := FcwbErrorMessage.cwbsvGetErrTextIndexed(errorIndex,errorText);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoErrorMessage.Create(ResultCodeText);
  Result := errorText
end;

{
Get top level error message.
}

function TcaoErrorMessage.GetErrMsg: string;
begin
  Result := GetErrTextIndexed(GetErrorCount);
end;

end.
