{
Client Access Objects 1.5

caoProgram component

(c) 1998, 1999 Steven P. Goldsmith
All Rights Reserved

The cwbProgram automation object is used to invoke a program on a remote AS/400
system.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 1, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

sgoldsmith@home.com
http://members.home.net/sgoldsmith/
}

unit caoProgram;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  caoErrorMessage, caoRemoteCallConversation,

  // Import type library "Client Access Object Library" before installing.

  Cwbautsv_TLB;

type

  // caoProgram exception object

  EcaoProgram = class(Exception);

  // TcaoProgram component

  TcaoProgram = class(TComponent)
  private
    { Private declarations }
    FReturnCode: Integer;
    FSetupCalled: Boolean;
    FprogramName: WideString;
    FlibraryName: WideString;
    FcwbProgram: cwbProgram;
    FcaoErrorMessage: TcaoErrorMessage;
    FcaoRemoteCallConversation: TcaoRemoteCallConversation;
    FOnCallPgm: TNotifyEvent;
    function GetProgramName: WideString;
    procedure SetProgramName(PgmName: WideString);
    function GetLibraryName: WideString;
    procedure SetLibraryName(LibName: WideString);
    function GetParameterCount: SmallInt;
  protected
    { Protected declarations }
    function ResultCodeText: String; virtual;
    function GetErrMsg: String; virtual;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Setup; virtual;
    procedure AddParm(type_: Smallint; length: Integer; parameter: OleVariant); virtual;
    procedure CallPgm; virtual;
    procedure GetParm(index: Smallint; var type_: Smallint; var length: Integer; var parameter: OleVariant); virtual;
    procedure SetParm(index, type_: Smallint; length: Integer; parameter: OleVariant); virtual;
  published
    { Published declarations }
    property ReturnCode: Integer read FReturnCode default cwbOK;
    property programName: WideString read GetProgramName write SetProgramName;
    property libraryName: WideString read GetLibraryName write SetLibraryName;
    property parameterCount: Smallint read GetParameterCount;
    property system: TcaoRemoteCallConversation read FcaoRemoteCallConversation write FcaoRemoteCallConversation;
    property errorMessage: TcaoErrorMessage read FcaoErrorMessage write FcaoErrorMessage;
    property OleObject: cwbProgram read FcwbProgram;
    property OnCallPgm: TNotifyEvent read FOnCallPgm write FOnCallPgm;
  end;

implementation

{
Create OLE object.
}

constructor TcaoProgram.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FSetupCalled := false;
  FcwbProgram := CocwbProgram.Create
end;

{
Return formatted result code text.
}

function TcaoProgram.ResultCodeText: String;
begin
  Result := Format('Result code %d',[FReturnCode])
end;

{
Return error mesage or formatted result code text.
}

function TcaoProgram.GetErrMsg: String;
var
  S: String;
begin
  S := FcaoErrorMessage.GetErrMsg;
  if S = '' then
    S := ResultCodeText;
  Result := S
end;

{
This property contains the name of the program to be run.  This is a read/write
property.

Get OLE property programName if setup was called.  Otherwise, get private
variable.
}

function TcaoProgram.GetProgramName: WideString;
begin
  if FSetupCalled then
    Result := FcwbProgram.programName
  else
    Result := FprogramName
end;

{
Set OLE property programName if setup was called. Otherwise, set private
variable.
}

procedure TcaoProgram.SetProgramName(PgmName: WideString);
begin
  if FSetupCalled then
    FcwbProgram.programName := PgmName
  else
    FprogramName := PgmName
end;

{
This property contains the name of the library in which the program exists.
This is a read/write property.

Get OLE property libraryName setup was called.  Otherwise, get private
variable.
}

function TcaoProgram.GetLibraryName: WideString;
begin
  if FSetupCalled then
    Result := FcwbProgram.libraryName
  else
    Result := FlibraryName
end;

{
Set OLE property libraryName if setup was called. Otherwise, set private
variable.
}

procedure TcaoProgram.SetLibraryName(LibName: WideString);
begin
  if FSetupCalled then
    FcwbProgram.libraryName := LibName
  else
    FlibraryName := LibName
end;

{
This property contains the number of parameters for this program object.  This
is a read-only property.
}

function TcaoProgram.GetParameterCount: SmallInt;
begin
  Result := FcwbProgram.parameterCount
end;

{
Handle caoErrorMessage or caoRemoteCallConversation component being removed.
}

procedure TcaoProgram.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FcaoErrorMessage then
      FcaoErrorMessage := nil
    else
      if AComponent = FcaoRemoteCallConversation then
        FcaoRemoteCallConversation := nil
  end
end;

{
This method initializes the cwbProgram object.  It must be called before any
other method or before accessing any of the objects properties.
}

procedure TcaoProgram.Setup;
begin
  FReturnCode := FcwbProgram.cwbrcSetup(FprogramName,FlibraryName);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoProgram.Create(ResultCodeText);
  FSetupCalled := true
end;

{
This method adds a parameter to the program object.  This method should be
called once for each parameter to be added.  When the program is called, the
parameters are passed in the same order in which they were added using this
method.

Parameters

type - Short Integer

The type of parameter to be added.  Use one of the defined parameter types:
cwbrcInput, cwbcOutput, or cwbrcInout.

This type can be ORed, with any of the following constants, to automatically
convert the parameter data between ASCII and EBCDIC:

cwbrcTextConvert

  All parameter data (input, output, or both) will be converted between EBCDIC
  and ASCII

cwbrcTextConvertInput

  Input parameter data will be converted from ASCII to EBCDIC.  This value is
  useful for a cwbrcInout parameter if only the input data should be converted.

cwbrcTextConvertOutput

  Output parameter data will be converted from EBCDIC to ASCII.  This value is
  useful for a cwbrcInout parameter if only the output data should be converted.

length - Long

The length of the input parameter, if the type is cwbrcInput or cwbrcInout.
If the type is cwbrcOutput, this parameter is ignored.

parameter - Byte Array Variant

Pointer to a buffer that  contains the value, if the type is cwbrcInput or
cwbrcInout.  If the type is cwbrcOutput, this parameter ignored.  (This
parameter should be passed as a SafeArray of unsigned characters.)
}

procedure TcaoProgram.AddParm(type_: Smallint; length: Integer; parameter: OleVariant);
begin
  FReturnCode := FcwbProgram.cwbrcAddParm(type_,length,parameter);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoProgram.Create(GetErrMsg)
end;

{
This method calls a specified program on an AS/400 system.
}

procedure TcaoProgram.CallPgm;
begin
  if (FcaoRemoteCallConversation <> nil) and
  (FcaoErrorMessage  <> nil) then
    FReturnCode := FcwbProgram.cwbrcCallPgm(FcaoRemoteCallConversation.OleObject,
    FcaoErrorMessage.OleObject);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoProgram.Create(GetErrMsg);
  if Assigned(OnCallPgm) then
    OnCallPgm(Self)
end;

{
This method retrieves information about a program parameter.

Parameters:
index - Short Integer

Identifies which parameter to retrieve.  This index can be from zero to the
total number of parameters less one.  The total number of parameters can be
accessed by retrieving the value of the parameterCount property.

type - Pointer to Short Integer

Pointer to a buffer that is used to store the parameter type.  The value will
be one of the defined parameter types:  cwbrcInput, cwbrcOutput, or
cwbrcInout, optionally Ored with cwbrcTextConvert, cwbrcTextConvertInput, or
cwbrcTextConvertOutput.

length - Pointer to Long

Pointer to a buffer that is used to store the length of the parameter value.

parameter - Byte Array Variant

Pointer to a buffer that is used to store the parameter value.  (This parameter
should be passed as a SafeArray of unsigned characters.)

Make sure pParm has been initilized to the proper size before calling.
}

procedure TcaoProgram.GetParm(index: Smallint; var type_: Smallint; var length: Integer; var parameter: OleVariant);
begin
  FReturnCode := FcwbProgram.cwbrcGetParm(index,type_,length,parameter);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoProgram.Create(GetErrMsg)
end;

{
This method changes the information for a program parameter.

Parameters:

index - Short Integer

Identifies which parameter to change.  This index can be from zero to the total
number of parameters less one.  The total number of parameters can be accessed
by retrieving the value of the parameterCount property.

type - Short Integer

The new parameter type.  Use one of the defined parameter types:  cwbrcInput,
cwbrcOutput, cwbrcInout.  This type can be ORed, with any of the following
constants, to automatically convert the parameter data between ASCII and
EBCDIC:

  cwbrcTextConvert

  All parameter data (input, output, or both) will be converted between EBCDIC
  and ASCII

  cwbrcTextConvertInput

  Input parameter data will be converted from ASCII to EBCDIC.  This value is
  useful for a cwbrcInout parameter if only the input data should be converted.

  cwbrcTextConvertOutput

  Output parameter data will be converted from EBCDIC to ASCII.  This value is
  useful for a cwbrcInout parameter if only the output data should be converted.

length - Long

The length of the input parameter, if the type is cwbrcInput or cwbrcInout.  If
the type is cwbrcOutput, this parameter is ignored.

parameter - Byte Array Variant

Pointer to a buffer that  contains the value, if the type is cwbrcInput or
cwbrcInout.  If the type is cwbrcOutput, this parameter ignored.  (This
parameter should be passed as a SafeArray of unsigned characters.)
}

procedure TcaoProgram.SetParm(index, type_: Smallint; length: Integer; parameter: OleVariant);
begin
  FReturnCode := FcwbProgram.cwbrcSetParm(index,type_,length,parameter);
  // Raise exception if needed.
  if FReturnCode <> cwbOK then
    raise EcaoProgram.Create(GetErrMsg)
end;

end.
